#!/usr/bin/perl

# A utility script to test the &parse_text routine.

# BUG: Requires a copy of &parse_text, doesn't use the implementation
# in wiki-to-latex.pl!

use strict;
use IO::File;

if ( $#ARGV != 1 ) {
    die "usage: $0 <src> <dst>\n";
}

my ($src, $dst) = @ARGV;

# process source

my $text;
my $src_fh = new IO::File;

$src_fh->open($src)
    or die "Failed to open file '$src' for reading: $!\n";

while ( <$src_fh> ) {
    $text .= $_;
}

$src_fh->close;

# process destination

my $dst_fh = new IO::File;

$dst_fh->open("> $dst")
    or die "Failed to open file '$dst' for writing: $!\n";

&parse_text($text, $dst_fh);

$dst_fh->close;

# subroutines

sub parse_text
{
    my ($text, $fh) = @_;

    my $ul_level = 0; # bullet list level
    my $ol_level = 0; # numbered list level
    my $dl_level = 0; # definition list level
    my $tabular = 0; # whether a table is being processed

    $text =~ s/\r//g; # weed \r from sources
    $text =~ s/_/\\_/g; # quote underscores

    foreach ( split /\n/, $text ) {	

	next if m|^//|;

	# sections
	if ( /^(={1,4})\s*(.+?)\s*={1,4}/ ) {
	    my $len = length $1;
	    $fh->printf("\\%ssection{%s}\n", 'sub' x $len, $2);
	}

	# bullet lists
	elsif ( /^(\*{1,4})\s*(.+)$/ ) {
	    my $level = length $1;
	    if ( $level > $ul_level ) {
                for (1 .. $level - $ul_level) {
                    $fh->print("\\begin{itemize}\n");
                }
	    }
	    elsif ( $level < $ul_level ) {
                for (1 .. $ul_level - $level) {
                    $fh->print("\\end{itemize}\n");
                }
	    }
	    $fh->printf("\\item %s\n", $2);
	    $ul_level = $level;
	}

	# numbered lists
	elsif ( /^(\#{1,4})\s*(.+)$/ ) {
	    my $level = length $1;
	    if ( $level > $ol_level ) {
                for (1 .. $level - $ol_level) {
                    $fh->print("\\begin{enumerate}\n");
                }
	    }
	    elsif ( $level < $ol_level ) {
                for (1 .. $ol_level - $level) {
                    $fh->print("\\end{enumerate}\n");
                }
	    }
	    $fh->printf("\\item %s\n", $2);
	    $ol_level = $level;
	}

	# definition lists
	elsif ( /^(;{1,4})(.+?):(.+)$/ ) {
	    my $level = length $1;
	    if ( $level > $dl_level ) {
                for (1 .. $level - $dl_level) {
                    $fh->print("\\begin{description}\n");
                }
	    }
	    elsif ( $level < $dl_level ) {
                for (1 .. $dl_level - $level) {
                    $fh->print("\\end{description}\n");
                }
	    }
	    $fh->printf("\\item[%s] %s\n", $2, $3);
	    $dl_level = $level;
	}

	# tables
	elsif ( /^\|\|/ ) {
	    my @columns = split /\s*\|\|\s*/;
	    shift @columns; # weed extra element due to leading ||

	    if ( not $tabular ) {
		$fh->printf("\\begin{tabular}{%s}\n", "l" x ($#columns+1));
		$tabular = 1;
	    }
	    $fh->printf("%s \\\\\n", join(" & ", @columns));
	}

	# regular text
	else {
	    while ( $ul_level ) { # standard, close bullet list
		$fh->print("\\end{itemize}\n");
		$ul_level--;
	    }

	    while ( $ol_level ) { # standard, close numbered list
		$fh->print("\\end{enumerate}\n");
		$ol_level--;
	    }

	    while ( $dl_level ) { # standard, close definition list
		$fh->print("\\end{description}\n");
		$dl_level--;
	    }

	    if ( $tabular ) { # close tabulating environment
		$fh->printf("\\end{tabular}\n");
		$tabular = 0;
	    }

	    # handle latex special characters

	    s/#/\\#/g;

	    # output
		
	    $fh->print("$_\n");
	}
    }

    while ( $ul_level ) { # end of document, close bullet list
	$fh->print("\\end{itemize}\n");
	$ul_level--;
    }

    while ( $ol_level ) { # end of document, close numbered list
	$fh->print("\\end{enumerate}\n");
	$ol_level--;
    }

    while ( $dl_level ) { # end of document, close definition list
	$fh->print("\\end{description}\n");
	$dl_level--;
    }
}
