#!/usr/bin/perl

use strict;
use Date::Parse;

# BEG: configurables

my $gnuplot = "gnuplot";
my $plot = "dht-activity-plot";

# END: configurables

if ( $#ARGV != 0 ) {
    printf STDERR "usage: $0 <png|ps>\n";
    exit 1;
}

my ($terminal) = @ARGV;

if ( $terminal eq 'ps' ) {
    $terminal = 'postscript';
    $plot = sprintf "%s.%s", $plot, "ps";
}
elsif ( $terminal eq 'png' ) {
    $terminal = 'png small color';
    $plot = sprintf "%s.%s", $plot, "png";
}
else {
    printf STDERR "usage: $0 <png|ps>\n";
    exit 1;
}

my %title = ( "anttisalonen" => "as",
	      "tuomastoivonen" => "toivotuo",
	      "simoviitanen" => "szviitan",
	      "markorih" => "mjraiha",
	      "ristosaarelma" => "rsaarelm",
	      "tomitukiainen" => "ttukiain" );

open GNUPLOT, "| $gnuplot"
    or die "Failed to open pipe to '$gnuplot': $!\n";

printf GNUPLOT "set out '%s'\n", $plot;
printf GNUPLOT "set terminal %s\n", $terminal;
printf GNUPLOT "set xrange [0:160]\n";
printf GNUPLOT "set yrange [0:320]\n";
printf GNUPLOT "min(a,b) = (a < b) ? a : b\n";
printf GNUPLOT "goal(x)=2.07*(x-19)\n";
my @datasets;

foreach my $file ( <*.txt> ) {
    my ($basename) = split /\./, $file;
    &process($basename);
    push @datasets, sprintf("'%s' title '%s' with lines",
			    "$basename.dat",
			    $title{$basename});
}
# (Antti Salonen) Added a graph for GOAL. Starting from day 19 an estimated
# 2.07 hours a day to achieve a total of 240 hours by May 14th.
push @datasets, sprintf ("min(goal(x),240) title 'GOAL' with lines");

printf GNUPLOT "plot %s\n", join(", ", @datasets);

close GNUPLOT;

sub process
{
    my $file = shift;

    my $in = "$file.txt";
    my $out = "$file.dat";

    my %worklog;
    my $total;

    open IN, $in
	or die "Failed to open file: $in: $!\n";

    my $name = <IN>; # first line has the person's name
    chomp($name);
    
    while ( <IN> ) {
	next if /^\s*$/; # weed empty lines
	next if /^#/; # weed comments
	
	my ($date, $type, $hours, $description) = split /\s+/, $_, 4;
	my ($day, $month, $year) = split /\./, $date;
	if ( length($day) == 1 ) { $day = "0$day"; }
	if ( length($month) == 1 ) { $month = "0$month"; }
	$worklog{"$year-$month-$day"} += $hours;
    }
    
    close IN;

    open OUT, "> $out"
	or die "Failed to open for writing file: $out: $!\n";

    foreach my $date ( sort keys %worklog ) {
	$total += $worklog{$date};
	my $time = str2time($date);
	my $yday = (localtime($time))[7];
	printf OUT "%s %s\n", $yday, $total;
    }

    close OUT;
}
