#!/usr/bin/perl

use strict;
use Date::Parse;

# BEG: configurables

my $gnuplot = "gnuplot";
my $plot = "dht-goal-plot";

# END: configurables

if ( $#ARGV != 0 ) {
    printf STDERR "usage: $0 <png|ps>\n";
    exit 1;
}

my ($terminal) = @ARGV;

if ( $terminal eq 'ps' ) {
    $terminal = 'postscript';
    $plot = sprintf "%s.%s", $plot, "ps";
}
elsif ( $terminal eq 'png' ) {
    $terminal = 'png small color';
    $plot = sprintf "%s.%s", $plot, "png";
}
else {
    printf STDERR "usage: $0 <png|ps>\n";
    exit 1;
}

my %title = ( "anttisalonen" => "as",
	      "tuomastoivonen" => "toivotuo",
	      "simoviitanen" => "szviitan",
	      "markorih" => "mjraiha",
	      "ristosaarelma" => "rsaarelm",
	      "tomitukiainen" => "ttukiain" );

open GNUPLOT, "| $gnuplot"
    or die "Failed to open pipe to '$gnuplot': $!\n";

printf GNUPLOT "set out '%s'\n", $plot;
printf GNUPLOT "set terminal %s\n", $terminal;
printf GNUPLOT "min(a,b) = (a < b) ? a : b\n";
printf GNUPLOT "goal(x)=2.07*(x-19)\n";
printf GNUPLOT "set yrange [:2]\n";
printf GNUPLOT "set xrange [15:160]\n";
#printf GNUPLOT "set xdata time\n";
#printf GNUPLOT "set timefmt \"%Y-%m-%d\"\n";
#printf GNUPLOT "set format x \"%b %Y\"\n";


my @datasets;

foreach my $file ( <*.txt> ) {
    my ($basename) = split /\./, $file;
    &process($basename);
    push @datasets, sprintf("'%s' title '%s' with lines",
			    "$basename.dat",
			    $title{$basename});
}
push @datasets, sprintf ("min(goal(x),240)/goal(x) title 'GOAL'");

# BEG: hack
#
#my $start = (localtime(str2time("2004-01-15")))[7];
#my $today = (localtime(time))[7];
#
#open GOAL, "> goal.dat"
#    or die "Failed to open for writing file: foo.dat: $!\n";
#
#foreach my $i ( $start .. $today ) {
#    printf GOAL "%d %d\n", $i, 1;
#}
#
#close GOAL;
#
#push @datasets, sprintf("'%s' title '%s' with lines",
#			"goal.dat",
#			"GOAL");
#
# END: hack

printf GNUPLOT "plot %s\n", join(", ", @datasets);

close GNUPLOT;

sub process
{
    my $file = shift;

    my $in = "$file.txt";
    my $out = "$file.dat";

    my %worklog;
    my $total;

    open IN, $in
	or die "Failed to open file: $in: $!\n";

    my $name = <IN>; # first line has the person's name
    chomp($name);
    
    while ( <IN> ) {
	next if /^\s*$/; # weed empty lines
	next if /^#/; # weed comments
	
	my ($date, $type, $hours, $description) = split /\s+/, $_, 4;
	my ($day, $month, $year) = split /\./, $date;
	if ( length($day) == 1 ) { $day = "0$day"; }
	if ( length($month) == 1 ) { $month = "0$month"; }
	$worklog{"$year-$month-$day"} += $hours;
    }
    
    close IN;

    open OUT, "> $out"
	or die "Failed to open for writing file: $out: $!\n";

    foreach my $date ( sort keys %worklog ) {
	$total += $worklog{$date};
	my $time = str2time($date);
	my $yday = (localtime($time))[7];
	my $goal = &goal($date);
#	my $percentage = 1;
#	if ( $total < $goal ) {
#	    $percentage = ($goal / $total) * -1;
#	}
#	if ( $total > $goal ) {
#	    $percentage = $total / $goal;
#	}
	my $percentage = $total / $goal;
	printf OUT "%d %f\n", $yday, $percentage;
    }

    close OUT;
}

sub goal
{
    my $date = shift;

    my $start = (localtime(str2time("2004-01-22")))[7];
    my $end = (localtime(str2time("2004-05-14")))[7];

    my $today = (localtime(str2time($date)))[7];

    my $days_total = ($end - $start) + 1;
    my $days_past = $today - $start;

    if ( $days_past <= 0 ) { $days_past = 1; }

    my $goal = ($days_past / $days_total) * 240;

    return $goal;
}
