#!/usr/bin/perl

use strict;
use Date::Parse;

if ( $#ARGV != 1 ) {
    printf STDERR "usage: $0 <start> <end>\n";
    exit 1;
}

my ($start, $end) = @ARGV;

my @people = ('anttisalonen', 'tuomastoivonen', 'simoviitanen',
	      'markorih', 'ristosaarelma', 'tomitukiainen');

my %types = ( "AT" => "aineistoon tutustuminen",
	      "YP" => "yllpito",
	      "PS" => "projektisuunnitelma",
	      "VA" => "vaatimusanalyysi",
	      "SU" => "suunnittelu",
	      "KO" => "koodaus",
	      "LR" => "loppuraportti" );

my %worklog;

foreach my $person ( @people ) {

    my $file = sprintf "../../tytunnit/%s.txt", "$person";

    open IN, $file
	or die "open failed: $file: $!\n";

    my $name = <IN>; # first line has the person's name
    chomp($name);
    
    while ( <IN> ) {
	next if /^\s*$/; # weed empty lines
	next if /^\#/; # weed comments

	my ($date, $type, $hours, $description) = split /\s+/, $_, 4;
	if ( $type eq 'OH' ) { $type = 'KO' }; # fix bugs in timesheets
	my ($day, $month, $year) = split /\./, $date;
	if ( length($day) == 1 ) { $day = "0$day"; }
	if ( length($month) == 1 ) { $month = "0$month"; }

	my $time = str2time("$year-$month-$day");
	my $yday = (localtime($time))[7];

	$worklog{$yday}{$type} += $hours;
    }
}

my $start_yday = (localtime(str2time($start)))[7];
my $end_yday = (localtime(str2time($end)))[7];

my (%period, %total);
my ($period_all, $total_all);

foreach my $yday ( sort keys %worklog ) {
    foreach my $type ( keys %{$worklog{$yday}} ) {
	if ( ($yday >= $start_yday) and ($yday <= $end_yday) ) {
	    $period{$type} += $worklog{$yday}{$type};
	    $period_all += $worklog{$yday}{$type};
	}
	$total{$type} += $worklog{$yday}{$type};
	$total_all += $worklog{$yday}{$type};
    }
}

print "\\begin{tabular}{lllll}\n";

printf("%s & %s & %s & %s & %s \\\\ \n",
       "\\bfseries Tyn luokittelu",
       "\\bfseries Tunniste",
       "\\bfseries Tunnit vaiheessa",
       "\\bfseries Tunnit yhteens",
       "\\bfseries Prosenttia");

print "\\hline\n";

foreach my $type ( keys %period ) {
    printf("%s & %s & %d h & %d h & %.1f \\%% \\\\ \n",
	   $types{$type}, $type, $period{$type}, $total{$type},
	   ($period{$type} / $total{$type})*100);
}

printf("%s & %s & %d h & %d h & %.1f \\%% \\\\ \n",
       'yhteens', "--", $period_all, $total_all,
       ($period_all / $total_all)*100);

print "\\end{tabular}\n";

my $pstart_yday = (localtime(str2time("2004-01-22")))[7];
my $pend_yday = (localtime(str2time("2004-05-24")))[7];

my $avg_activity = $total_all / (($pend_yday - $pstart_yday)+1);

my $period_length = (($end_yday - $start_yday)+1);
my $period_activity = $period_all / $period_length;

print "\\begin{description}\n";

printf("\\item[%s] %.2f h\n", 
       "Keskimrin tunteja per piv koko projektin aikana",
       $avg_activity);

printf("\\item[%s] %.2f h\n", 
       "Keskimrin tunteja per piv projektivaiheen aikana",
       $period_activity);

printf("\\item[%s] %.2f\n", 
       "Projektivaiheen aktiivisuus koko projektiin verrattuna",
       ($period_activity / $avg_activity));

print "\\end{description}\n";
